@@ -7,13 +7,18 @@
 #' \code{grid.size}) at each point. It then returns summary statistics for the
 #' range of AOOs calculated, and the RasterLayer(s) containing the grids with
 #' the minimum AOO. It is the base function which is used by
-#' \code{gridUncertainty} and \code{gridUncertaintySimulation}
+#' \code{gridUncertainty}, \code{gridUncertaintySimulation}, and
+#' \code{gridUncertaintyRestricted}
 #' @inheritParams createGrid
 #' @param splits Specifies the number of ways to split the grid in ONE axis.
 #' @param min.percent.rule Logical. If \code{TRUE}, a minimum area threshold
 #'   must be passed before a grid is counted as an AOO grid.
 #' @param percent Numeric. The minimum percent to be applied as a threshold for
 #'   the \code{min.percent.rule}.
+#' @param restriction Logical. If \code{TRUE}, allows user to specify areas to
+#'   focus where grid search is done. Used in gridUncertaintyRestricted.
+#' @param min.grids.shift Dataframe object with two columns (x.shift and
+#'   y.shift) specifying the coordinates to restrict the AOO grid placement.
 #' @return List containing the following:
 #' \itemize{
 #'  \item Vector of length split*split of calculated AOO for each shifted grid
@@ -26,16 +31,12 @@
 #'   \email{calvinkflee@@gmail.com}
 #' @family gridUncertainty functions
 #' @seealso \code{\link{createGrid}} \code{\link{getAOOSilent}}
-#' @references Bland, L.M., Keith, D.A., Miller, R.M., Murray, N.J. and
-#'   Rodriguez, J.P. (eds.) 2016. Guidelines for the application of IUCN Red
-#'   List of Ecosystems Categories and Criteria, Version 1.0. Gland,
-#'   Switzerland: IUCN. ix + 94pp. Available at the following web site:
-#'   \url{iucnrle.org/}
 #' @import raster
 
-gridUncertaintyBase <- function(ecosystem.data, grid.size,
-                                splits, min.percent.rule = TRUE, percent = 1){
-  grid <- createGrid(ecosystem.data, grid.size)
+gridUncertaintyBase <- function(input.data, grid.size,
+                                splits, min.percent.rule = FALSE, percent = 1,
+                                restriction = FALSE, min.grids.shift){
+  grid <- createGrid(input.data, grid.size)
   intervals <- grid.size/splits
 
   x.shift <- seq(0, grid.size, intervals)
@@ -46,6 +47,22 @@ gridUncertaintyBase <- function(ecosystem.data, grid.size,
   shift.grid <- expand.grid(x.shift = (x.shift + sample(0:(grid.size*0.05), 1)),
                             y.shift = (y.shift + sample(0:(grid.size*0.05), 1)))
 
+  if(restriction){ # Only modift shift.grid when restriction = T
+    # apply to get index of the restricted shift.grid we want
+    shift.grid.restricted.index <- apply(min.grids.shift, 1, function(grids){
+      min.AOO.x <- grids['x.shift']
+      x.lims <- c(min.AOO.x - intervals, min.AOO.x + intervals)
+      min.AOO.y <- grids['y.shift']
+      y.lims <- c(min.AOO.y - intervals, min.AOO.y + intervals)
+      index <- which(x.lims[1] < shift.grid$x.shift &
+                       shift.grid$ x.shift < x.lims[2] &
+                       y.lims[1] < shift.grid$y.shift &
+                       shift.grid$y.shift< y.lims[2])
+      return(index)
+      shift.grid.restricted.index <- unique(unlist(shift.grid.restricted.index))
+      shift.grid <- shift.grid[shift.grid.restricted.index, ]
+    })
+  }
   grid.shifted.list <- apply(shift.grid, 1, function(gridList){
     # Pull out the values for each shift
     current.xshift <- gridList["x.shift"]
@@ -53,9 +70,8 @@ gridUncertaintyBase <- function(ecosystem.data, grid.size,
     grid.shift <- shift(grid, x = current.xshift, y = current.yshift)
     return(grid.shift)
   })
-
   AOO.list <- lapply(grid.shifted.list, getAOOSilent, # List of AOO for each scenario
-                     ecosystem.data = ecosystem.data,
+                     input.data = input.data,
                      min.percent.rule = min.percent.rule,
                      percent = percent)
   AOO.numbers <- list()
@@ -101,16 +117,11 @@ gridUncertaintyBase <- function(ecosystem.data, grid.size,
 #'   \email{calvinkflee@@gmail.com}
 #' @family gridUncertainty functions
 #' @seealso \code{\link{createGrid}} \code{\link{getAOOSilent}}
-#' @references Bland, L.M., Keith, D.A., Miller, R.M., Murray, N.J. and
-#'   Rodriguez, J.P. (eds.) 2016. Guidelines for the application of IUCN Red
-#'   List of Ecosystems Categories and Criteria, Version 1.0. Gland,
-#'   Switzerland: IUCN. ix + 94pp. Available at the following web site:
-#'   \url{iucnrle.org/}
 #' @import raster
 
-gridUncertaintyRandomManual <- function(ecosystem.data, grid.size,
-                                      n.sim = 10, min.percent.rule = T, percent = 1){
-  grid <- createGrid(ecosystem.data = ecosystem.data, grid.size = grid.size)
+gridUncertaintyRandomManual <- function(input.data, grid.size, n.sim = 10,
+                                        min.percent.rule = FALSE, percent = 1){
+  grid <- createGrid(input.data = input.data, grid.size = grid.size)
   results.df <- data.frame(sim.no = integer(),
                            x.shift = integer(),
                            y.shift = integer(),
@@ -123,7 +134,7 @@ gridUncertaintyRandomManual <- function(ecosystem.data, grid.size,
     y.shift <- sample(-grid.size:grid.size, 1)
     dist.move <- sqrt((x.shift^2)+(y.shift^2)) # Total distance moved using Pythagoras
     shifted.grid <- shift(grid, x = x.shift, y = y.shift)
-    AOO <- getAOOSilent(ecosystem.data = ecosystem.data,
+    AOO <- getAOOSilent(input.data = input.data,
                         grid = shifted.grid, min.percent.rule = min.percent.rule,
                         percent = percent) # get the AOO for each sampled grid
     sim.df <- data.frame(sim.no = i,
@@ -177,11 +188,6 @@ gridUncertaintyRandomManual <- function(ecosystem.data, grid.size,
 #'   \email{murr.nick@@gmail.com}
 #' @family gridUncertainty functions
 #' @seealso \code{\link{createGrid}} \code{\link{getAOOSilent}}
-#' @references Bland, L.M., Keith, D.A., Miller, R.M., Murray, N.J. and
-#'   Rodriguez, J.P. (eds.) 2016. Guidelines for the application of IUCN Red
-#'   List of Ecosystems Categories and Criteria, Version 1.0. Gland,
-#'   Switzerland: IUCN. ix + 94pp. Available at the following web site:
-#'   \url{iucnrle.org/}
 #' @examples
 #' crs.UTM55S <- '+proj=utm +zone=55 +south +ellps=WGS84 +datum=WGS84 +units=m +no_defs'
 #' r1 <- raster(ifelse((volcano<130), NA, 1), crs = crs.UTM55S)
@@ -191,10 +197,10 @@ gridUncertaintyRandomManual <- function(ecosystem.data, grid.size,
 #' @export
 #' @import raster
 
-gridUncertaintyRandom <- function(ecosystem.data, grid.size,
-                                  n.AOO.improvement, min.percent.rule = T,
-                                  percent = 1, max.n.rounds = 1000){
-  grid <- createGrid(ecosystem.data = ecosystem.data, grid.size = grid.size)
+gridUncertaintyRandom <- function(input.data, grid.size, n.AOO.improvement,
+                                  min.percent.rule = FALSE, percent = 1,
+                                  max.n.rounds = 1000){
+  grid <- createGrid(input.data = input.data, grid.size = grid.size)
   results.df <- data.frame(sim.no = integer(),
                            x.shift = integer(),
                            y.shift = integer(),
@@ -207,7 +213,7 @@ gridUncertaintyRandom <- function(ecosystem.data, grid.size,
     y.shift <- sample(-grid.size:grid.size, 1)
     dist.move <- sqrt((x.shift^2)+(y.shift^2)) # Total distance moved using Pythagoras
     shifted.grid <- shift(grid, x = x.shift, y = y.shift)
-    AOO <- getAOOSilent(ecosystem.data = ecosystem.data,
+    AOO <- getAOOSilent(input.data = input.data,
                         grid = shifted.grid, min.percent.rule = min.percent.rule,
                         percent = percent) # get the AOO for each sampled grid
     sim.df <- data.frame(sim.no = i,
@@ -225,7 +231,7 @@ gridUncertaintyRandom <- function(ecosystem.data, grid.size,
     y.shift <- sample(-grid.size:grid.size, 1)
     dist.move <- sqrt((x.shift^2)+(y.shift^2)) # Total distance moved using Pythagoras
     shifted.grid <- shift(grid, x = x.shift, y = y.shift)
-    AOO <- getAOOSilent(ecosystem.data = ecosystem.data,
+    AOO <- getAOOSilent(input.data = input.data,
                         grid = shifted.grid, min.percent.rule = min.percent.rule,
                         percent = percent) # get the AOO for each sampled grid
     sim.df <- data.frame(sim.no = i,
@@ -272,7 +278,6 @@ gridUncertaintyRandom <- function(ecosystem.data, grid.size,
 #'   must be passed before a grid is counted as an AOO grid.
 #' @param percent Numeric. The minimum percent to be applied as a threshold for
 #'   the \code{min.percent.rule}.
-#' @param max.rounds Specifies the max number of rounds to run the function.
 #' @return A list containing the following:
 #' \itemize{
 #'  \item Data frame of results showing the minimum AOO calculated for each
@@ -282,35 +287,32 @@ gridUncertaintyRandom <- function(ecosystem.data, grid.size,
 #' }
 #' @author Calvin Lee \email{calvinkflee@@gmail.com}
 #' @family gridUncertainty functions
-#' @references Bland, L.M., Keith, D.A., Miller, R.M., Murray, N.J. and
-#'   Rodriguez, J.P. (eds.) 2016. Guidelines for the application of IUCN Red
-#'   List of Ecosystems Categories and Criteria, Version 1.0. Gland,
-#'   Switzerland: IUCN. ix + 94pp. Available at the following web site:
-#'   \url{iucnrle.org/}
 #' @examples
 #' crs.UTM55S <- '+proj=utm +zone=55 +south +ellps=WGS84 +datum=WGS84 +units=m +no_defs'
 #' r1 <- raster(ifelse((volcano<130), NA, 1), crs = crs.UTM55S)
 #' extent(r1) <- extent(0, 6100, 0, 8700)
 #' x <- gridUncertainty(r1, grid.size = 10000, n.AOO.improvement = 5,
-#'                            min.percent.rule = TRUE, percent = 1)
+#'                            min.percent.rule = FALSE, percent = 1)
 #' @export
 
-gridUncertainty <- function(ecosystem.data, grid.size, n.AOO.improvement,
-                            min.percent.rule = T, percent = 1, max.rounds = 1000){
+gridUncertainty <- function(input.data, grid.size, n.AOO.improvement,
+                            min.percent.rule = FALSE, percent = 1){
   out.df <- data.frame()
   min.grids.shift <- list()
   for (i in 1:n.AOO.improvement){ # First runs before checking for improvement
     out.df[i, 1] <- i
-    results <- gridUncertaintyBase(ecosystem.data = ecosystem.data,
+    results <- gridUncertaintyBase(input.data = input.data,
                                    grid.size = grid.size, splits = i,
                                    min.percent.rule = min.percent.rule,
                                    percent = percent)
     out.df[i, 2] <- results$summary.df$min.AOO
     min.grids.shift[[i]] <- results$min.AOO.shifts
   }
-  for (i in n.AOO.improvement+1:max.rounds){ #arbitrary large number
+  i <- n.AOO.improvement + 1
+  logic.test <- FALSE
+  while(all(logic.test) == FALSE){
     out.df[i, 1] <- i
-    results <- gridUncertaintyBase(ecosystem.data = ecosystem.data,
+    results <- gridUncertaintyBase(input.data = input.data,
                                    grid.size = grid.size, splits = i,
                                    min.percent.rule = min.percent.rule,
                                    percent = percent)
@@ -321,7 +323,7 @@ gridUncertainty <- function(ecosystem.data, grid.size, n.AOO.improvement,
       logic.test <- c(logic.test, out.df[(i-n.AOO.improvement), 2] <=
                         out.df[(i-n.AOO.improvement+j), 2])
     }
-    if (all(logic.test)) break # Stop the function when AOO no longer decreases
+    i <- i + 1
   }
   names(out.df) <- c('n.splits', 'min.AOO')
   # Find splits which generated the smallest AOOs
@@ -333,11 +335,11 @@ gridUncertainty <- function(ecosystem.data, grid.size, n.AOO.improvement,
   min.AOO.y.shift <- min.grids.shift[[min.AOO.split.n]]$y.shift[1]
 
   # Recreate this AOO grid
-  original.grid <- createGrid(ecosystem.data, grid.size)
+  original.grid <- createGrid(input.data, grid.size)
   min.AOO.grid.shifts <- shift(original.grid,
                                min.AOO.x.shift,
                                min.AOO.y.shift)
-  min.AOO.grid <- getAOOSilent(ecosystem.data, min.AOO.grid.shifts,
+  min.AOO.grid <- getAOOSilent(input.data, min.AOO.grid.shifts,
                                min.percent.rule = min.percent.rule, percent = percent)
 
   results.list <- list('min.AOO.df' = out.df,
@@ -361,20 +363,15 @@ gridUncertainty <- function(ecosystem.data, grid.size, n.AOO.improvement,
 #'   for each grid shift scenario.
 #' @author Calvin Lee \email{calvinkflee@@gmail.com}
 #' @family gridUncertainty functions
-#' @references Bland, L.M., Keith, D.A., Miller, R.M., Murray, N.J. and
-#'   Rodriguez, J.P. (eds.) 2016. Guidelines for the application of IUCN Red
-#'   List of Ecosystems Categories and Criteria, Version 1.0. Gland,
-#'   Switzerland: IUCN. ix + 94pp. Available at the following web site:
-#'   \url{iucnrle.org/}
 
-gridUncertaintySimulation <- function(ecosystem.data, grid.size,
-                                      simulations, min.percent.rule = T, percent = 1){
+gridUncertaintySimulation <- function(input.data, grid.size, simulations,
+                                      min.percent.rule = FALSE, percent = 1){
   out.df <- data.frame('n.splits' = rep(0, simulations),
                        'min.AOO' = rep(0, simulations),
                        'max.AOO' = rep(0, simulations))
   for(i in 1:simulations){
     out.df[i, 1] <- i
-    results <- gridUncertaintyBase(ecosystem.data = ecosystem.data,
+    results <- gridUncertaintyBase(input.data = input.data,
                                grid.size = grid.size, splits = i,
                                min.percent.rule = min.percent.rule, percent = percent)
     out.df[i, 2] <- results$stats$min.AOO
@@ -382,3 +379,93 @@ gridUncertaintySimulation <- function(ecosystem.data, grid.size,
   }
   return(out.df)
 }
+
+#' Function to compute AOO with grid uncertainty systematically with stopping
+#' rule and restrictions
+#'
+#' \code{gridUncertaintyRestricted} determines the number of area of occupancy (AOO) grid
+#' cells occupied by a species or ecosystem systematically. It will only stop
+#' when the AOO calculated does not improve (decrease) after a set number of
+#' split scenarios. The number of grids within each split is restricted to only
+#' include those which are already found nearby to ones already with the minimum
+#' AOO.
+#' @inheritParams gridUncertainty
+#' @return A list containing the following:
+#' \itemize{
+#'  \item Data frame of results showing the minimum AOO calculated for each
+#'  shift scenario
+#'  \item Single SpatialPolygonsDataFrame containing the AOO grid which would
+#'  produce the minimum AOO calculated
+#' }
+#' @author Calvin Lee \email{calvinkflee@@gmail.com}
+#' @family gridUncertainty functions
+
+gridUncertaintyRestricted <- function(input.data, grid.size, n.AOO.improvement,
+                                      min.percent.rule = FALSE, percent = 1){
+  out.df <- data.frame()
+  min.grids.shift.list <- list()
+  n.shifts <- vector()
+  for (i in 1:n.AOO.improvement){ # First runs before checking for improvement
+    out.df[i, 1] <- i
+    results <- gridUncertaintyBase(input.data = input.data,
+                                   grid.size = grid.size, splits = i,
+                                   min.percent.rule = min.percent.rule,
+                                   percent = percent, restriction = FALSE)
+    out.df[i, 2] <- results$summary.df$min.AOO
+    min.grids.shift.list[[i]] <- results$min.AOO.shifts
+    n.shifts <- c(n.shifts, results$summary.df$n.shifts)
+    logic.test <- vector()
+    for (j in 1:(n.AOO.improvement-1)){
+      logic.test <- c(logic.test, out.df[(i-n.AOO.improvement), 2] <=
+                        out.df[(i-n.AOO.improvement+j), 2])
+    }
+  }
+
+  i <- n.AOO.improvement + 1
+  logic.test <- FALSE
+  while(all(logic.test) == FALSE){
+    # Find grids which generated the smallest AOOs
+    min.AOO.split.index <- which(out.df[, 2] == min(out.df[, 2]))
+    min.grids.shift <- data.frame()
+    for(j in 1:length(min.AOO.split.index)){
+      min.grids.shift <- rbind(min.grids.shift, min.grids.shift.list[[j]])
+    }
+    out.df[i, 1] <- i
+    results <- gridUncertaintyBase(input.data = input.data,
+                                   grid.size = grid.size, splits = i,
+                                   min.percent.rule = min.percent.rule,
+                                   percent = percent,
+                                   restriction = T,
+                                   min.grids.shift = min.grids.shift)
+    out.df[i, 2] <- results$summary.df$min.AOO
+    min.grids.shift.list[[i]] <- results$min.AOO.shifts
+    n.shifts <- c(n.shifts, results$summary.df$n.shifts)
+    logic.test <- vector()
+    for (j in 1:(n.AOO.improvement-1)){
+      logic.test <- c(logic.test, out.df[(i-n.AOO.improvement), 2] <=
+                        out.df[(i-n.AOO.improvement+j), 2])
+    }
+    i <- i + 1
+  }
+
+  names(out.df) <- c('n.splits', 'min.AOO')
+  # Only need to report one grid which resulted in true min.AOO
+  min.AOO.split.n <- min(min.AOO.split.index)
+
+  # Get one the shifts needed to get this min AOO
+  min.AOO.x.shift <- min.grids.shift.list[[min.AOO.split.n]]$x.shift[1]
+  min.AOO.y.shift <- min.grids.shift.list[[min.AOO.split.n]]$y.shift[1]
+
+  # Recreate this AOO grid
+  original.grid <- createGrid(input.data, grid.size)
+  min.AOO.grid.shifts <- shift(original.grid,
+                               min.AOO.x.shift,
+                               min.AOO.y.shift)
+  min.AOO.grid <- getAOOSilent(input.data, min.AOO.grid.shifts,
+                               min.percent.rule = min.percent.rule, percent = percent)
+
+  results.list <- list('min.AOO.df' = out.df,
+                       'min.AOO.grid' = min.AOO.grid,
+                       'total shifts' = sum(n.shifts))
+  return(results.list)
+}
