@@ -5,7 +5,7 @@ test_that("input cannot be in lonlat crs",{
   expect_error(getArea(r.crs), "Input raster has a longitude/latitude CRS.\nPlease reproject to a projected coordinate system")
 })
 
-test_that("accepts different input types", {
+test_that("accepts different input rasters", {
   # Dummy rasters for testing
   r <- raster(nrows=10, ncols=10)
   crs(r) <- "+proj=utm +zone=55 +south +datum=WGS84 +units=m +no_defs +ellps=WGS84 +towgs84=0,0,0"
@@ -19,6 +19,28 @@ test_that("accepts different input types", {
   expect_equal(getArea(r.multiple, 1), 0.01296)
 })
 
+test_that("output is a single value", {
+  # Dummy raster
+  r <- raster(nrows=10, ncols=10)
+  crs(r) <- "+proj=utm +zone=55 +south +datum=WGS84 +units=m +no_defs +ellps=WGS84 +towgs84=0,0,0"
+  values(r) <- rep(c(1, NA), 50)
+
+  expect_equal(length(getArea(r)), 1)
+})
+
+test_that("accepts SpatialPolygons", {
+  # Dummy rectangle
+  x1 = 0
+  x2 = 1000
+  y1 = 0
+  y2 = 1000
+  my_polygon = Polygon(cbind(c(x1,x1,x2,x2,x1),c(y1,y2,y2,y1,y1)))
+  my_polygons = Polygons(list(my_polygon), ID = "A")
+  dummy_polygon = SpatialPolygons(list(my_polygons))
+
+  expect_equal(getArea(dummy_polygon), 1)
+})
+
 context("Change functions")
 
 test_that("decline stats work", {
@@ -31,6 +53,6 @@ test_that("decline stats work", {
                                       methods = c('ARD', 'PRD', 'ARC'))
   expect_equal(dummy.decline.df$ARD, 10)
   expect_equal(dummy.decline.df$PRD, 12.94494, tolerance=1e-5)
-  expect_equal(dummy.decline.df$ARC, -0.1386294, tolerance=1e-5)
+  expect_equal(dummy.decline.df$ARC, -13.86294, tolerance=1e-5)
   expect_error(getDeclineStats(A.t1, A.t2, year.t1, year.t2))
 })
